
import { createRoot } from 'react-dom/client'
import React, { lazy, Suspense, useEffect } from 'react'
import './index.css'

// Log initialisation de l'application
console.log('Initialisation de l\'application - ' + new Date().toISOString());

// Dynamic import of the App component
const App = lazy(() => import('./App'));

// Create a loading component with skeleton UI
const LoadingFallback = () => (
  <div className="w-full h-screen flex items-center justify-center bg-climat-light">
    <div className="flex flex-col items-center">
      <div className="w-16 h-16 border-4 border-climat-turquoise border-solid rounded-full border-t-transparent animate-spin mb-4"></div>
      <div className="text-climat-turquoise font-medium">Chargement...</div>
    </div>
  </div>
)

// Define proper interface for font preload links
interface FontPreloadLink {
  href: string;
  as: string;
  type: string;
  crossOrigin?: string;
}

// Define proper interface for CSS preload links
interface CssPreloadLink {
  href: string;
  as: string;
}

// Type for all preload links
type PreloadLink = FontPreloadLink | CssPreloadLink;

// Add preload links for important resources
const addPreloadLinks = () => {
  // Preload critical fonts
  const fontPreloadLinks: FontPreloadLink[] = [
    { href: '/fonts/inter-var.woff2', as: 'font', type: 'font/woff2', crossOrigin: 'anonymous' },
  ];

  // Preload critical CSS
  const cssPreloadLinks: CssPreloadLink[] = [
    { href: '/src/index.css', as: 'style' },
  ];

  // Combine all links
  const allLinks: PreloadLink[] = [...fontPreloadLinks, ...cssPreloadLinks];

  // Add all links to the document head
  allLinks.forEach(linkData => {
    const link = document.createElement('link');
    link.rel = 'preload';
    link.href = linkData.href;
    link.as = linkData.as;
    
    // Handle optional properties with type checking
    if ('type' in linkData && linkData.type) link.type = linkData.type;
    if ('crossOrigin' in linkData && linkData.crossOrigin) link.crossOrigin = linkData.crossOrigin;
    
    document.head.appendChild(link);
  });
};

// Force refresh when service worker detects a new version, but only in production
const setupServiceWorkerUpdateNotifier = () => {
  // Only in production, not in development/preview
  if ('serviceWorker' in navigator && !window.location.host.includes('lovableproject.com')) {
    // Handle updates from service worker
    navigator.serviceWorker.addEventListener('message', (event) => {
      if (event.data && event.data.type === 'UPDATE_AVAILABLE') {
        console.log('Nouvelle version disponible. Rechargement...');
        // Add a small delay to ensure the message is seen
        setTimeout(() => {
          window.location.reload();
        }, 1000);
      }
    });
    
    // Check for updates less frequently - once every 30 minutes instead of 5
    setInterval(() => {
      navigator.serviceWorker.ready.then(registration => {
        registration.update();
      });
    }, 1800000); // 30 minutes
  }
};

// Vérifie que le script GPT Engineer est chargé
const checkGPTEngScript = () => {
  const scripts = document.getElementsByTagName('script');
  let hasGPTEngScript = false;
  for (let i = 0; i < scripts.length; i++) {
    if (scripts[i].src.includes('gptengineer.js')) {
      hasGPTEngScript = true;
      break;
    }
  }
  console.log('Script GPT Engineer détecté: ' + hasGPTEngScript);
  if (!hasGPTEngScript) {
    console.warn('ATTENTION: Script GPT Engineer non détecté. La fonctionnalité de preview pourrait ne pas fonctionner correctement.');
  }
};

// Register service worker for production with improved handling
const registerServiceWorker = () => {
  // Only register service worker in production, not in development/preview
  if ('serviceWorker' in navigator && !window.location.host.includes('lovableproject.com')) {
    window.addEventListener('load', () => {
      // Use a timestamped service worker URL to bypass the browser cache
      const swUrl = `/service-worker.js?v=${new Date().getTime()}`;
      
      navigator.serviceWorker.register(swUrl)
        .then(registration => {
          console.log('SW registered successfully:', registration);
          
          registration.addEventListener('updatefound', () => {
            const newWorker = registration.installing;
            console.log('New service worker installing...');
            
            if (newWorker) {
              newWorker.addEventListener('statechange', () => {
                if (newWorker.state === 'installed') {
                  if (navigator.serviceWorker.controller) {
                    console.log('New content available. Please refresh.');
                    // Only force refresh in production environment
                    if (!window.location.host.includes('lovableproject.com')) {
                      setTimeout(() => window.location.reload(), 1000);
                    }
                  } else {
                    console.log('Content is cached for offline use.');
                  }
                }
              });
            }
          });
        })
        .catch(error => {
          console.error('SW registration failed:', error);
        });
    });
    
    // Set up the update notifier
    setupServiceWorkerUpdateNotifier();
  } else {
    console.log('Service worker not registered in development/preview environment');
  }
};

// Clear browser cache on startup (only when forced, not in preview)
const clearBrowserCache = () => {
  // Skip cache clearing in preview environment
  if (window.location.host.includes('lovableproject.com')) {
    return;
  }
  
  if ('caches' in window) {
    caches.keys().then(cacheNames => {
      cacheNames.forEach(cacheName => {
        if (cacheName.includes('climat-distinction')) {
          console.log('Clearing cache:', cacheName);
          caches.delete(cacheName);
        }
      });
    });
  }
};

// Optimize resource loading
const optimizeResourceLoading = () => {
  // Add preload links
  addPreloadLinks();
  
  // Vérifier le script GPT Engineer 
  checkGPTEngScript();

  // Clear cache in development or when forced, but not in preview
  if ((window.location.search.includes('force_refresh=true') || window.location.hostname === 'localhost') 
      && !window.location.host.includes('lovableproject.com')) {
    clearBrowserCache();
  }

  // Register service worker for production only, not for preview
  if (!window.location.host.includes('lovableproject.com')) {
    registerServiceWorker();
  } else {
    console.log('Skipping service worker registration in preview environment');
  }

  // Log du démarrage
  console.log('Configuration initiale terminée - ' + new Date().toISOString());
};

// Call optimization functions
optimizeResourceLoading();

// Prefetch on hover for faster navigation
document.addEventListener('mouseover', (e) => {
  const target = e.target as HTMLElement;
  const linkElem = target.closest('a');
  
  if (linkElem && linkElem.href && linkElem.href.startsWith(window.location.origin)) {
    const url = new URL(linkElem.href);
    
    // Don't prefetch if it's the current page or already prefetched
    if (url.pathname !== window.location.pathname && !linkElem.dataset.prefetched) {
      const prefetchLink = document.createElement('link');
      prefetchLink.rel = 'prefetch';
      prefetchLink.href = url.href;
      document.head.appendChild(prefetchLink);
      
      linkElem.dataset.prefetched = 'true';
      console.log('Prefetching:', url.href);
    }
  }
}, { passive: true });

// Ensure the root element only has one React root
const rootElement = document.getElementById("root");
if (rootElement) {
  try {
    // Use a type-safe approach instead of checking internal React property
    const existingRoot = (rootElement as any).__reactRoot;
    
    if (!existingRoot) {
      createRoot(rootElement).render(
        <React.StrictMode>
          <Suspense fallback={<LoadingFallback />}>
            <App />
          </Suspense>
        </React.StrictMode>
      );
      console.log('Application rendue avec succès - ' + new Date().toISOString());
    } else {
      console.log('React root already exists, skipping render');
    }
  } catch (e) {
    console.error('Erreur lors du rendu de l\'application:', e);
  }
} else {
  console.error('Élément root non trouvé dans le DOM');
}
